function hboxeq=showeqbox(f,opt)
%SHOWEQBOX   Show the equation box of a fit.
%   H = SHOWEQBOX(F) shows the equation box of the fit F, using the settings
%   defined in FITPARAM. The fit structure F is obtained from FIT. H is a
%   handle to the equation box.
%
%   Note that SHOWEQBOX interprets the greek symbols in the fitting equation
%   (latex syntax). The '\' latex symbol must be omitted.
%
%   SHOWEQBOX is automatically called from SHOWFIT or SELECTFIT when the
%   option 'dispeqboxmode' is set to 'on' in the fitparam.m file.
%
%   Example:
%      plotsample('power')
%      f = fit('alpha/x^n');
%      showeqbox(f);
%
%   See also FITPARAM, FIT, SHOWFIT, SELECTFIT, DISPEQFIT.

%   F. Moisy, moisy_at_fast.u-psud.fr
%   Revision: 1.11,  Date: 2006/09/06
%   This function is part of the EzyFit Toolbox

% History:
% 2006/02/08: v1.00, first version.
% 2006/02/14: v1.10, compatible with 'y(x)=..' (free function name).
%                    Capital greek letters accepted. Greek letters as
%                    subscript accepted.
% 2006/09/06: v1.11, bug fixed when the handle f.hdata is invalid


if exist('fitparam.m','file'),
    fp=fitparam;
else
    error('No fitparam file found.');
end;

if isfield(f,'eq'), % for normal fits (no interpolation):
    
    streq = [f.yvar '(' f.xvar ') = ' f.eq];  
    
    if strcmp(fp.eqdisplaymode,'replace'),
        for n=1:length(f.m),
            streq = strrep(streq, f.param{n}, num2str(f.m(n), fp.numberofdigit));
        end;
        if length(streq)>fp.maxlengtheq,
            streq=[streq(1:fp.maxlengtheq) '...'];
        end;
        streq=greekize(streq);
        streq={streq};
    else
        % truncates the equation string if too long:
        if length(streq)>fp.maxlengtheq,
            streq=[streq(1:fp.maxlengtheq) '...'];
        end;
        streq={greekize(streq)};
        for n=1:length(f.m),
            strm = [greekize(f.param{n}) ' = ' num2str(f.m(n), fp.numberofdigit)];
            streq = {streq{:} strm};
        end;
    end;

    lastline='';
    switch lower(fp.corrcoefmode),
        case 'r', lastline=['R = ' num2str(f.r, fp.numberofdigit) '  '];
        case 'r2', lastline=['R^2 = ' num2str(f.r^2, fp.numberofdigit) '  '];
    end;
    if strcmp(fp.linlogdisp,'on');
        lastline=[lastline '(' f.fitmode ')'];
    end;
    if length(lastline)
        streq = {streq{:} lastline};
    end;
    
else
    streq=f.name; % for interpolations
end;



% number of fit already present in the figure:
numann=length(findall(gcf,'UserData','equationbox'));
% position of the new annotation textbox, to avoid overlaping:
position = fp.boxlocation+numann*[0.01 -0.01 0 0];

% determines the box color:
if (~isfield(f,'hdata')),
    fitcolor=[0 0 0]; % default color if not data in the figure
else
    if length(fp.fitcolor)==3
        fitcolor=fp.fitcolor; % fixed color
    else
        if ishandle(f.hdata)
            co=get(f.hdata); % object properties of the data
            fitcolor=max(0,min(1,co.Color*fp.fitcolor)); % color indexed from that of the data
        else
            fitcolor=[0 0 0];
        end;
    end;
end;

bgcolor='white';
if nargin>1,
    if strcmp(opt,'transparent'), % option used with 'selectfit', in order not to hide the data during the selection
        bgcolor='none';
    end;
end;

hboxeq=annotation('textbox',position,'BackgroundColor',bgcolor,...
    'Margin',3,'Color',fitcolor,'EdgeColor',fitcolor,...
    'FitHeightToText','on','FitHeightToText','on',...
    'UserData','equationbox','String',streq);
